'use strict';

module.exports.make = function make(ntest) {
    // Hoisted variables
    var vows, assert, nfs, mockDir, suite, deps;

    // Dependecy injection and variable initializations
    vows    = ntest.vows;
    assert  = ntest.assert;
    nfs     = ntest.nfs;
    
    mockDir = __dirname + '/mock/lib/util/file';
    suite   = vows.describe('File util test suite');

    // Batch 1 - testing the walk() method
    suite.addBatch({
        'GIVEN the mock dir in "test/mock/lib/util/file" WHEN walk() is called' : {
            topic : function topic() {
                nfs.make().walk(mockDir, this.callback);
            },

            'THEN "root1.js" should exists' : function vow(err, files) {
                assert.include(files, 'root1.js');
            },

            'THEN "root2.js" should exists' : function vow(err, files) {
                assert.include(files, 'root2.js');
            },

            'THEN ".ignore" should not exist' : function vow(err, files) {
                assert.isUndefined(files['.ignore']);
            },

            'THEN ".ignoredir/happy.js" should not exist' : function vow(err, files) {
                assert.isUndefined(files['.ignoredir/happy.js']);
            },

            'THEN "sub1/sub1-1.js" should exist' : function vow(err, files) {
                assert.include(files, 'sub1/sub1-1.js');
            },

            'THEN "sub2/sub2-1.js" should exist' : function vow(err, files) {
                assert.include(files, 'sub2/sub2-1.js');
            },

            'THEN "sub2/sub22/sub22-1.js" should exist' : function vow(err, files) {
                assert.include(files, 'sub2/sub22/sub22-1.js');
            }
        }
    });

    // Batch 2 - test multiple calls of walk() to ensure they are sandboxed properly
    suite.addBatch({
        'GIVEN the mock dir in "test/mock/lib/util/file"' : {
            topic : function topic() {
                return nfs.make();
            },

            'WHEN walk() is called [1]' : {
                topic : function topic(nfs) {
                    nfs.walk(mockDir, this.callback);
                },

                'THEN it should contain exactly 5 files' : function vow(err, files) {
                    assert.equal(files.length, 5);
                }
            },

            'WHEN walk() is called [2]' : {
                topic : function topic(nfs) {
                    nfs.walk(mockDir, this.callback);
                },

                'THEN it should contain exactly 5 files' : function vow(err, files) {
                    assert.equal(files.length, 5);
                }
            },

            'WHEN walk() is called [3]' : {
                topic : function topic(nfs) {
                    nfs.walk(mockDir, this.callback);
                },

                'THEN it should contain exactly 5 files' : function vow(err, files) {
                    assert.equal(files.length, 5);
                }
            },

            'WHEN walk() is called [4]' : {
                topic : function topic(nfs) {
                    nfs.walk(mockDir, this.callback);
                },

                'THEN it should contain exactly 5 files' : function vow(err, files) {
                    assert.equal(files.length, 5);
                }
            },

            'WHEN walk() is called [5]' : {
                topic : function topic(nfs) {
                    nfs.walk(mockDir, this.callback);
                },

                'THEN it should contain exactly 5 files' : function vow(err, files) {
                    assert.equal(files.length, 5);
                }
            }
        }
    })

    return suite;
}